"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isArrayBuffer = isArrayBuffer;
exports.isBigIntTypedArray = isBigIntTypedArray;
exports.isCanonicalIntegerIndexString = isCanonicalIntegerIndexString;
exports.isIterable = isIterable;
exports.isObject = isObject;
exports.isObjectLike = isObjectLike;
exports.isOrdinaryArray = isOrdinaryArray;
exports.isOrdinaryTypedArray = isOrdinaryTypedArray;
exports.isSharedArrayBuffer = isSharedArrayBuffer;
exports.isTypedArray = isTypedArray;

var _primordials = require("./primordials.cjs");

/**
 * @param {unknown} value
 * @returns {value is {}}
 */
function isObject(value) {
  return value !== null && typeof value === "object" || typeof value === "function";
}
/**
 * @param {unknown} value
 * @returns {value is {}}
 */


function isObjectLike(value) {
  return value !== null && typeof value === "object";
} // Inspired by util.types implementation of Node.js

/** @typedef {Uint8Array|Uint8ClampedArray|Uint16Array|Uint32Array|Int8Array|Int16Array|Int32Array|Float32Array|Float64Array|BigUint64Array|BigInt64Array} TypedArray */

/**
 * @param {unknown} value
 * @returns {value is TypedArray}
 */


function isTypedArray(value) {
  return (0, _primordials.TypedArrayPrototypeGetSymbolToStringTag)(value) !== undefined;
}
/**
 * @param {unknown} value
 * @returns {value is BigInt64Array|BigUint64Array}
 */


function isBigIntTypedArray(value) {
  const typedArrayName = (0, _primordials.TypedArrayPrototypeGetSymbolToStringTag)(value);
  return typedArrayName === "BigInt64Array" || typedArrayName === "BigUint64Array";
}
/**
 * @param {unknown} value
 * @returns {value is ArrayBuffer}
 */


function isArrayBuffer(value) {
  try {
    (0, _primordials.ArrayBufferPrototypeGetByteLength)(
    /** @type {any} */
    value);
    return true;
  } catch (e) {
    return false;
  }
}
/**
 * @param {unknown} value
 * @returns {value is SharedArrayBuffer}
 */


function isSharedArrayBuffer(value) {
  if (_primordials.NativeSharedArrayBuffer === null) {
    return false;
  }

  try {
    (0, _primordials.SharedArrayBufferPrototypeGetByteLength)(
    /** @type {any} */
    value);
    return true;
  } catch (e) {
    return false;
  }
}
/**
 * @param {unknown} value
 * @throws {TypeError}
 * @returns {value is Iterable<unknown>}
 */


function isIterable(value) {
  return typeof value[_primordials.SymbolIterator] === "function";
}
/**
 * @param {unknown} value
 * @returns {value is unknown[]}
 */


function isOrdinaryArray(value) {
  if (!(0, _primordials.ArrayIsArray)(value)) {
    return false;
  }

  const iterator = value[_primordials.SymbolIterator]();

  if (iterator[_primordials.SymbolToStringTag] !== "Array Iterator") {
    return false;
  }

  return true;
}
/**
 * @param {unknown} value
 * @returns {value is TypedArray}
 */


function isOrdinaryTypedArray(value) {
  if (!isTypedArray(value)) {
    return false;
  }

  const iterator = value[_primordials.SymbolIterator]();

  if (iterator[_primordials.SymbolToStringTag] !== "Array Iterator") {
    return false;
  }

  return true;
}
/**
 * @param {unknown} value
 * @returns {value is string}
 */


function isCanonicalIntegerIndexString(value) {
  if (typeof value !== "string") {
    return false;
  }

  const number = (0, _primordials.NativeNumber)(value);

  if (value !== number + "") {
    return false;
  }

  if (!(0, _primordials.NumberIsFinite)(number)) {
    return false;
  }

  if (number !== (0, _primordials.MathTrunc)(number)) {
    return false;
  }

  return true;
}