"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.IsDetachedBuffer = IsDetachedBuffer;
exports.LengthOfArrayLike = LengthOfArrayLike;
exports.SpeciesConstructor = SpeciesConstructor;
exports.ToIntegerOrInfinity = ToIntegerOrInfinity;
exports.defaultCompare = defaultCompare;

var _is = require("./is.cjs");

var _messages = require("./messages.cjs");

var _primordials = require("./primordials.cjs");

/**
 * @see https://tc39.es/ecma262/#sec-tointegerorinfinity
 * @param {unknown} target
 * @returns {number}
 */
function ToIntegerOrInfinity(target) {
  if (typeof target === "bigint") {
    throw (0, _primordials.NativeTypeError)(_messages.CANNOT_CONVERT_A_BIGINT_VALUE_TO_A_NUMBER);
  }

  const number = (0, _primordials.NativeNumber)(target);

  if ((0, _primordials.NumberIsNaN)(number) || number === 0) {
    return 0;
  }

  return (0, _primordials.MathTrunc)(number);
}
/**
 * @see https://tc39.es/ecma262/#sec-tolength
 * @param {unknown} target
 * @returns {number}
 */


function ToLength(target) {
  const length = ToIntegerOrInfinity(target);

  if (length < 0) {
    return 0;
  }

  return length < _primordials.NativeNumber.MAX_SAFE_INTEGER ? length : _primordials.NativeNumber.MAX_SAFE_INTEGER;
}
/**
 * @see https://tc39.es/ecma262/#sec-lengthofarraylike
 * @param {object} arrayLike
 * @returns {number}
 */


function LengthOfArrayLike(arrayLike) {
  if (!(0, _is.isObject)(arrayLike)) {
    throw (0, _primordials.NativeTypeError)(_messages.THIS_IS_NOT_AN_OBJECT);
  }

  return ToLength(
  /** @type {any} */
  arrayLike.length);
}
/**
 * @see https://tc39.es/ecma262/#sec-speciesconstructor
 * @param {object} target
 * @param {{ new(...args: any[]): any; }} defaultConstructor
 * @returns {{ new(...args: any[]): any; }}
 */


function SpeciesConstructor(target, defaultConstructor) {
  if (!(0, _is.isObject)(target)) {
    throw (0, _primordials.NativeTypeError)(_messages.THIS_IS_NOT_AN_OBJECT);
  }

  const constructor = target.constructor;

  if (constructor === undefined) {
    return defaultConstructor;
  }

  if (!(0, _is.isObject)(constructor)) {
    throw (0, _primordials.NativeTypeError)(_messages.THE_CONSTRUCTOR_PROPERTY_VALUE_IS_NOT_AN_OBJECT);
  }

  const species = constructor[_primordials.SymbolSpecies];

  if (species == null) {
    return defaultConstructor;
  }

  return species;
}
/**
 * @see https://tc39.es/ecma262/#sec-isdetachedbuffer
 * @param {ArrayBufferLike} buffer
 * @returns {boolean}
 */


function IsDetachedBuffer(buffer) {
  try {
    (0, _primordials.ArrayBufferPrototypeSlice)(buffer, 0, 0);
    return false;
  } catch (e) {
    /* empty */
  }

  if (_primordials.NativeSharedArrayBuffer !== null) {
    try {
      (0, _primordials.SharedArrayBufferPrototypeSlice)(
      /** @type {SharedArrayBuffer} */
      buffer, 0, 0);
      return false;
    } catch (e) {
      /* empty */
    }
  }

  return true;
}
/**
 * bigint comparisons are not supported
 *
 * @see https://tc39.es/ecma262/#sec-%typedarray%.prototype.sort
 * @param {number} x
 * @param {number} y
 * @returns {-1 | 0 | 1}
 */


function defaultCompare(x, y) {
  const isXNaN = (0, _primordials.NumberIsNaN)(x);
  const isYNaN = (0, _primordials.NumberIsNaN)(y);

  if (isXNaN && isYNaN) {
    return 0;
  }

  if (isXNaN) {
    return 1;
  }

  if (isYNaN) {
    return -1;
  }

  if (x < y) {
    return -1;
  }

  if (x > y) {
    return 1;
  }

  if (x === 0 && y === 0) {
    const isXPlusZero = (0, _primordials.ObjectIs)(x, 0);
    const isYPlusZero = (0, _primordials.ObjectIs)(y, 0);

    if (!isXPlusZero && isYPlusZero) {
      return -1;
    }

    if (isXPlusZero && !isYPlusZero) {
      return 1;
    }
  }

  return 0;
}